@@ -15,6 +15,9 @@ except:
     IS_NEW = False
 
 
+from OFS.PropertyManager import PropertyManager
+from OFS.Traversable import Traversable
+
 from Testing import ZopeTestCase as ztc
 
 from Products.Five import zcml
@@ -29,6 +32,7 @@ from quintagroup.canonicalpath.interfaces import ICanonicalPath
 from quintagroup.canonicalpath.interfaces import ICanonicalLink
 from quintagroup.canonicalpath.adapters import PROPERTY_PATH
 from quintagroup.canonicalpath.adapters import PROPERTY_LINK
+from quintagroup.canonicalpath.upgrades import CanonicalConvertor
 
 class TestCase(ptc.PloneTestCase):
     class layer(PloneSite):
@@ -83,7 +87,6 @@ class TestIndexerRegistration(TestCase):
         
 class TestDefaultCanonicalPathAdapter(TestCase):
 
-
     def afterSetUp(self):
         self.loginAsPortalOwner()
         self.purl = getToolByName(self.portal, 'portal_url')
@@ -289,12 +292,167 @@ class TestDefaultCanonicalLinkAdapter(TestCase):
             "After deleted Canonical link customization property not set to "
             "default value for the portal")
 
+##
+## Dummy object for converter tests
+##
+class PortalURL:
+    def __call__(self):
+        return "http://nohost/plone"
+    def getRelativeContentPath(self, context):
+        return ("plone", context.getId())
+
+class BaseItem:
+    portal_url = PortalURL()
+
+    def __init__(self, id):
+        self.id = id
+
+    def getId(self):
+        return self.id
+
+    def absolute_url(self):
+        return self.portal_url() + '/'+ self.getId()
+
+class GoodItem(BaseItem, PropertyManager, Traversable):
+    """Property provider."""
+
+class NotPropertyProviderItem(BaseItem, Traversable):
+    """Not property provider."""
+
+class NotAdaptableItem(BaseItem):
+    """Not adaptable object."""
+
+class TestConvertor(unittest.TestCase):
+
+    def setUp(self):
+        self.convertor = CanonicalConvertor("http://domain.com")
+
+    def test_convertIPathToLink(self):
+        item = GoodItem("item")
+        item._setProperty(PROPERTY_PATH, "/www/some/path")
+        self.convertor.convertIPathToLink(item)
+        # 1. check canonical link in result object
+        result = ICanonicalLink(item).canonical_link
+        expect = "http://domain.com/www/some/path"
+        self.assertEqual(result, expect, "Got %s canonical link, " \
+                         "expect: %s" % (result, expect))
+        # 2. canonical path propery mast be delete from the object
+        self.assertEqual(item.hasProperty(ICanonicalPath(item).prop), False,
+                         "canonical path' property not deleted from the object")
+
+    def test_convertPPathToLink(self):
+        item = GoodItem("item")
+        item._setProperty("custom_property", "/www/some/path")
+        self.convertor.convertPPathToLink(item, prop="custom_property")
+        # 1. check canonical link in result object
+        result = ICanonicalLink(item).canonical_link
+        expect = "http://domain.com/www/some/path"
+        self.assertEqual(result, expect, "Got %s canonical link, " \
+                         "expect: %s" % (result, expect))
+        # 2. custom_property mast be deleted from the object
+        self.assertEqual(item.hasProperty("custom_property"), False,
+                         "custom_property not deleted from the object")
+
+    def test_convertBadItems(self):
+        bad = NotPropertyProviderItem("item")
+        self.convertor.convertIPathToLink(bad)
+        result = self.convertor.getLogs()
+        expect = "ERROR: exceptions.AttributeError: " \
+                 "NotPropertyProviderItem instance has no attribute 'hasProperty'"
+        self.assertEqual(expect in result, True, "Wrong log: %s" % result)
+
+        bad = NotAdaptableItem("item")
+        self.convertor.convertIPathToLink(bad)
+        result = self.convertor.getLogs()
+        expect = "ERROR: zope.component.interfaces.ComponentLookupError: "
+        self.assertEqual(expect in result, True, "Wrong log: %s" % result)
+
+    def test_loggingSuccess(self):
+        good = GoodItem("item")
+        self.convertor.convertIPathToLink(good)
+        result = self.convertor.getLogs()
+        expect = "SUCCESS"
+        self.assertEqual(expect in result, True, "Wrong log: %s" % result)
+
+    def test_loggingGet(self):
+        # log must collect new errors
+        # and return full log anytime
+        bad = NotPropertyProviderItem("item")
+        self.convertor.convertIPathToLink(bad)
+        logs = self.convertor.getLogs()
+        logs2 = self.convertor.getLogs()
+        assert logs != ""
+        self.assertEqual(logs == logs2, True,
+             "logs not equal: \"%s\" != \"%s\"" % (logs, logs2))
+        self.convertor.convertIPathToLink(bad)
+        logs3 = self.convertor.getLogs()
+        self.assertEqual(logs3 > logs2, True,
+             "Log was not updated - last: \"%s\", previous: \"%s\"" % (logs3, logs2))
+        
+    def test_loggingCleanup(self):
+        bad = NotPropertyProviderItem("item")
+        self.convertor.convertIPathToLink(bad)
+        assert self.convertor.getLogs() != ""
+        self.convertor.cleanupLogs()
+        logs = self.convertor.getLogs()
+        self.assertEqual(logs, "", "Log not cleand-up: \"%s\"" % logs)
+
+
+class TestAdaptersRegistration(unittest.TestCase):
+    """Test of default adapters registration."""
+
+    def setUp(self):
+        self.cant = "Can't get \"%s\" adapter for object, which implement: "
+        self.doget = "Get \"%s\" adapter for object, which implement: "
+
+    def test_PropertyManagerAndTraversable(self):
+        class ProperyAndTraverseProvider(BaseItem, PropertyManager, Traversable):
+            """Property and Traversable provider."""
+        item = ProperyAndTraverseProvider("test")
+        self.assertNotEqual(queryAdapter(item, ICanonicalLink), None,
+           self.cant % ICanonicalLink.__name__ + "IPropertyManager and ITraversable.")
+        self.assertNotEqual(queryAdapter(item, ICanonicalPath), None,
+           self.cant % ICanonicalPath.__name__ + "IPropertyManager and ITraversable.")
+
+    def test_Traversable(self):
+        """Traversable enough for get adapter"""
+        class TraverseProvider(BaseItem, Traversable):
+            """Only Traversable provider."""
+        item = TraverseProvider("test")
+        self.assertNotEqual(queryAdapter(item, ICanonicalLink), None,
+           self.cant % ICanonicalLink.__name__ + "only ITraversable.")
+        self.assertNotEqual(queryAdapter(item, ICanonicalPath), None,
+           self.cant % ICanonicalPath.__name__ + "only ITraversable.")
+
+    def test_PropertyManager(self):
+        """Implementing only IPropertyManager - not enough, because of
+           the default value need ITraversable.
+        """
+        class PropertyManagerProvider(BaseItem, PropertyManager):
+            """Only PropertyManager provider."""
+        item = PropertyManagerProvider("test")
+        self.assertEqual(queryAdapter(item, ICanonicalLink), None,
+           self.doget % ICanonicalLink.__name__ + "only IPropertyManager.")
+        self.assertEqual(queryAdapter(item, ICanonicalPath), None,
+           self.doget % ICanonicalPath.__name__ + "only IPropertyManager.")
+
+    def test_NotProvider(self):
+        class NotProvider(BaseItem):
+            """Nor Traversable not PropertyManager provider."""
+        item = NotProvider("test")
+        self.assertEqual(queryAdapter(item, ICanonicalLink), None,
+           self.doget % ICanonicalLink.__name__ + "nor ITraversabe not IPropertyManager.")
+        self.assertEqual(queryAdapter(item, ICanonicalPath), None,
+           self.doget % ICanonicalPath.__name__ + "nor ITraversabe not IPropertyManager.")
+
 
 def test_suite():
     return unittest.TestSuite([
         unittest.makeSuite(TestIndexerRegistration),
         unittest.makeSuite(TestDefaultCanonicalPathAdapter),
         unittest.makeSuite(TestDefaultCanonicalLinkAdapter),
+        unittest.makeSuite(TestConvertor),
+        unittest.makeSuite(TestAdaptersRegistration),
         ])
 
 if __name__ == '__main__':
